import logging

class NullHandler(logging.Handler):
    def emit(self, record):
        pass

def get_log(name):
    logger = logging.getLogger(name)
    logger.addHandler(NullHandler())
    return logger

